/*
 * Copyright (C) 2003 Philip Blundell <philb@gnu.org>
 * Copyright (C) 2005 Ross Burton <ross@openedhand.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <gdk/gdkx.h>

/* Duration in milliseconds for a hold to last to be a right click */
#define TIMEOUT 300
/* Maximum distance the cursor can move in pixels before a hold is a drag */
#define THRESHOLD 4

struct {
  guint timeout;
  GdkDisplay *gdisplay;
  GdkEvent ev;
  gint x, y;
} t;

static gboolean
timeout (gpointer p)
{
  if (t.timeout) {
    gint x, y;
    gdk_display_get_pointer (t.gdisplay, NULL, &x, &y, NULL);
    if ((abs (x - t.x) < THRESHOLD) && (abs (y - t.y) < THRESHOLD)) {
      t.ev.button.time += TIMEOUT;
      t.ev.button.button = 3;
      t.ev.button.type = GDK_BUTTON_PRESS;
      gtk_main_do_event (&t.ev);
      t.ev.button.type = GDK_BUTTON_RELEASE;
      gtk_main_do_event (&t.ev);
    } else {
      gtk_main_do_event (&t.ev);
    }
  }
  
  t.timeout = 0;
  return FALSE;
}

static void
filter (GdkEvent *ev, gpointer p)
{
  if (ev->type == GDK_BUTTON_PRESS || ev->type == GDK_BUTTON_RELEASE) {
    if ((ev->button.state & GDK_MOD3_MASK) && ev->button.button == 1) {
      ev->button.button = 3;
      gtk_main_do_event (ev);
      return;
    }
    
    if (ev->type == GDK_BUTTON_PRESS && ev->button.button == 1 && t.timeout == 0) {
      t.timeout = g_timeout_add (TIMEOUT, timeout, NULL);
      t.ev = *ev; /* TODO: use gdk_event_copy i guess */
      t.gdisplay = gdk_x11_lookup_xdisplay (GDK_WINDOW_XDISPLAY (ev->any.window));
      gdk_display_get_pointer (t.gdisplay, NULL, &t.x, &t.y, NULL);
    } else if (ev->type == GDK_BUTTON_RELEASE && ev->button.button == 1 && t.timeout) {
      g_source_remove (t.timeout);
      t.timeout = 0;
      gtk_main_do_event (&t.ev);
      gtk_main_do_event (ev);
    } else {
      gtk_main_do_event (ev);
    }
  } else {
      gtk_main_do_event (ev);
  }
}

void
gtk_module_init (gint *argc, gchar ***argv)
{
  t.timeout = 0;
  gdk_event_handler_set (filter, NULL, NULL);
}
